#!/usr/bin/perl -w
#                              -*- Mode: CPerl -*-

# Author: Arne Muller (arne.muller@gmail.com), (c) 2002 - 2012
# query.cgi part of TeXMed

use CGI qw(:standard);
use LWP::UserAgent;
use entrez;
use texmedconfig qw($proxy $web_view $tmp $max_fetch $email $version);
$| = 1;

print header(-target=>'query');
print start_html(-style=> {src => ['../texmed.css']},
                 -title=>"TeXMed Article Query",
                 -text=>'black',
                 -bgcolor=>'white'), "\n";


print "<table border=0>\n",
       "<tr>\n",
       '<td><a href="/"><img src="/images/logo.png" ',
      'alt="Bioinformatics.Org" border="0"></a></td>',"\n",
       '<td><h1>TeXMed - a BibTeX interface for PubMed<br></td></h>',
       "</tr>\n</table>\n";

print "<p class=note>[ <b>Version $version</b>, for suggestions and comments please contact ", em(a({-href=>"mailto:$email"}, "Arne Muller")), ', <a href="http://www.bioinformatics.org/ftp/pub/texmed/">download TeXMed</a>, <a target="new" href="http://www.emacswiki.org/emacs/TeXMed.el">Emacs mode for TeXMed</a> ]</p>';

print "The query gets submitted to NCBI PubMed. The chosen articles can 
then be exported (converted) in BibTeX format.\n", a({-href=>"../texmedhelp.html", 
-target=>'view'}, "Get Help ..."), p();

### get and process parameters
my $session = param('session');
my $query = param('query');

print
  start_multipart_form(-target=>'query', -name=>'queryform'),
  hidden(-name=>'session', -value=>$session),
  textfield(-name=>'query',
	    -default=>'',
	    -maxlength=>10000,
	    -size=>40), '&nbsp;'x2,"\n",
  submit(-value=>'query'), '&nbsp;'x2,"\n",
  button(-name=>'clear',
         -onclick=>'document.forms[0].query.value=""; document.forms[0].query.focus(); return true'), ' ',
  a({-href=>"$web_view/query/static/help/pmhelp.html", -target=>'tips'},
    " ", " Search tips"), ' & ',
  a({-href=>"http://www.ncbi.nlm.nih.gov/books/bv.fcgi?rid=helppubmed.section.pubmedhelp.Search_Field_Descrip", -target=>'tips'},
    'search field/tag descriptions'), br(),
end_form();

print small('query might be: "sternberg mj" "MacCallum" [au] bioinformatics [ta]'), p();

if ( $query ) {
  ### fetch from PubMed
  my $ua = new LWP::UserAgent();
  $ua->proxy(['http'] => $proxy) if ( $proxy );

  my @uids = entrez::get_pmids($ua, $query);
  print b("Received ", scalar(@uids), "references<p>");
  unless ( @uids ) {
    print "<p class=warn>NO references found!</p>\n";
    print end_html();
    exit();
  }
  sleep 1; # avoid NCBI flooding and getting blocked!

  my @refs = entrez::get_references($ua, @uids);
  if ( @refs != @uids ) {
     my $diff = @uids - @refs;
     print "<p class=warn>WARNING: could't fetch entries for $diff referneces!\n</p>";
  }

  print
    start_multipart_form(-target=>'view', -name => 'storeform',
			 action=>'list.cgi'),
      button(-name=>'select all', -onclick=>qq|
              var f = document.getElementsByName('PMID');
              for ( var i=0; i<f.length; i++ ) { f[i].checked = true; }
        |), ' ',
      button(-name=>'un-select all', -onclick=>qq|
              var f = document.getElementsByName('PMID');
	      for ( var i=0; i<f.length; i++ ) { f[i].checked = false;}
        |), ' ',
      hidden(-name=>'action', -value=>'store'),
      hidden(-name=>'session', -value=>$session),
      submit(-value=>'export'),
      checkbox(-name=>'abstract', -label=>" incl. abstract",
               onclick=>'toggleBibLink("abstract", "abstract"); return true;'),
      checkbox(-name=>'linkOut',
               onclick=>'toggleBibLink("linkOut", "linkOut"); return true;',
               -label=> " link article ids (requires \\usepackage{hyperref})"),
      p(),"\n";

  my $entry;
  my $n = 0;

  foreach my $ref ( @refs ) {
     my $uid = $ref->{PMID};
     my $text = summary($ref);
     my $link = a({-href=>"$web_view?Db=pubmed&Cmd=ShowDetailView&uid=$uid",
                   -target=>'view'},
                  $uid);
     $n++;
     my $export = a({-href=>"list.cgi?PMID=$uid", id => "pmid:$uid",
                     -target=>'view', -name => 'bibLink'},
                    'bibtex');
     my $type = join(', ', grep { $_ } @{$ref->{PublicationType}});
     $type = "[ $type ]" if ( $type );
     print "$n. ", checkbox(-name=>'PMID', -value => $uid),
       " $link, $export $type<br>\n$text<p>";
  }
  print end_form;
}

sub summary {
   my $ref = shift;

   my $jrl = $ref->{JournalTitle} ? $ref->{JournalTitle} : 'no journal title';
   my $vol = join(', ', grep { $_ }
                  $ref->{Volume},$ref->{Issue},$ref->{Number});
   my $year =  $ref->{Year} ? $ref->{Year} : 'no year';
   my $title= $ref->{ArticleTitle} ? $ref->{ArticleTitle} : 'no title';
   $title =~ s/\.$//;
   my $auth = join(', ', map { "$ref->{LastName}[$_], $ref->{Initials}[$_]"
                            } 0..@{$ref->{LastName}}-1);
   my $pages = $ref->{Pages} ? $ref->{Pages} : 'np page given';

   return "$auth <b>($year)</b>. $title. <em>$jrl</em>, $vol:$pages.";
}

### some javascript to add export of abstract + linkout to bibtex links
print qq|
  <script language="javascript">

  function toggleBibLink(checkboxType, action) {
     var elems = document.getElementsByName('bibLink');
     var j = 0;
     if ( document.storeform[checkboxType].checked ) {
        for ( var i=0; i<elems.length; i++ ) {
           elems[i].href = elems[i].href + "&" + action;
        }
     } else {
        for ( var i=0; i<elems.length; i++ ) {
           elems[i].href = elems[i].href.replace("&" + action, "");
        }
     }
  }

  toggleBibLink('abstract', 'abstract');
  toggleBibLink('linkOut', 'linkOut');
  </script>
|;


print end_html();
